@@ -18,10 +18,8 @@
 # MA 02110-1301 USA.
 #
 
-# pylint: disable=E0611
 from gi.repository import Gtk
 from gi.repository import Gdk
-# pylint: enable=E0611
 
 import logging
 
@@ -168,8 +166,7 @@ class vmmCreateInterface(vmmGObjectUI):
     @staticmethod
     def iface_in_use_by(conn, name):
         use_str = ""
-        for i in conn.list_interface_names():
-            iface = conn.get_interface(i)
+        for iface in conn.list_interfaces():
             if name in iface.get_slave_names():
                 if use_str:
                     use_str += ", "
@@ -511,8 +508,8 @@ class vmmCreateInterface(vmmGObjectUI):
                                         phys.address]
 
         row_dict = {}
-        for name in self.conn.list_interface_names():
-            iface = self.conn.get_interface(name)
+        for iface in self.conn.list_interfaces():
+            name = iface.get_name()
             key = iface.get_xmlobj()
             iface_type = iface.get_type()
             active = iface.is_active()
@@ -764,7 +761,7 @@ class vmmCreateInterface(vmmGObjectUI):
     def build_ip_info(self):
         def build_ip(addr_str):
             if not addr_str:
-                return None, None
+                raise ValueError(_("Please enter an IP address"))
             ret = addr_str.rsplit("/", 1)
             address = ret[0]
             prefix = None
@@ -775,7 +772,7 @@ class vmmCreateInterface(vmmGObjectUI):
         is_manual = self.widget("ip-do-manual").get_active()
 
         copy_row = uiutil.get_list_selection(
-            self.widget("ip-copy-interface-combo"))
+            self.widget("ip-copy-interface-combo"), None)
 
         v4_mode = self.widget("ipv4-mode").get_active()
         v4_addr = self.widget("ipv4-address").get_text()
@@ -818,9 +815,10 @@ class vmmCreateInterface(vmmGObjectUI):
                 if not ipv6.dhcp:
                     if v6_gate:
                         ipv6.gateway = v6_gate
-                    addr, prefix = build_ip(v4_addr)
-                    if addr:
-                        ipv6.add_ip(addr, prefix)
+                    for v6_addr in v6_addrlist:
+                        addr, prefix = build_ip(v6_addr)
+                        if addr:
+                            ipv6.add_ip(addr, prefix)
 
         return [is_manual, copy_name, ipv4, ipv6, proto_xml]
 
