@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2008, 2013 Red Hat, Inc.
+# Copyright (C) 2006-2008, 2013-2014 Red Hat, Inc.
 # Copyright (C) 2006 Daniel P. Berrange <berrange@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,10 @@
 
 import logging
 
-# pylint: disable=E0611
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
-# pylint: enable=E0611
 
 from virtinst import util
 
@@ -118,7 +116,7 @@ class vmmManager(vmmGObjectUI):
 
         self.ignore_pause = False
 
-        # Mapping of VM UUID -> tree model rows to
+        # Mapping of rowkey -> tree model rows to
         # allow O(1) access instead of O(n)
         self.rows = {}
 
@@ -194,12 +192,6 @@ class vmmManager(vmmGObjectUI):
         self.enable_polling(COL_NETWORK)
         self.enable_polling(COL_MEM)
 
-        # Select first list entry
-        vmlist = self.widget("vm-list")
-        if len(vmlist.get_model()) != 0:
-            vmlist.get_selection().select_iter(
-                vmlist.get_model().get_iter_first())
-
         # Queue up the default connection detector
         self.idle_emit("add-default-conn")
 
@@ -241,7 +233,7 @@ class vmmManager(vmmGObjectUI):
         self.hostcpucol = None
         self.netcol = None
 
-        self.vmmenu.destroy()  # pylint: disable=E1101
+        self.vmmenu.destroy()
         self.vmmenu = None
         self.connmenu.destroy()
         self.connmenu = None
@@ -427,7 +419,7 @@ class vmmManager(vmmGObjectUI):
     ##################
 
     def current_row(self):
-        return uiutil.get_list_selection(self.widget("vm-list"))
+        return uiutil.get_list_selection(self.widget("vm-list"), None)
 
     def current_vm(self):
         row = self.current_row()
@@ -447,12 +439,6 @@ class vmmManager(vmmGObjectUI):
         else:
             return handle.conn
 
-    def current_vmuuid(self):
-        vm = self.current_vm()
-        if vm is None:
-            return None
-        return vm.get_uuid()
-
     def current_conn_uri(self, default_selection=False):
         vmlist = self.widget("vm-list")
         model = vmlist.get_model()
@@ -506,7 +492,7 @@ class vmmManager(vmmGObjectUI):
             return
 
         if vm:
-            self.emit("action-show-domain", conn.get_uri(), vm.get_uuid())
+            self.emit("action-show-domain", conn.get_uri(), vm.get_connkey())
         else:
             if not self.open_conn():
                 self.emit("action-show-host", conn.get_uri())
@@ -517,7 +503,7 @@ class vmmManager(vmmGObjectUI):
         if vm is None:
             self._do_delete_conn(conn)
         else:
-            self.emit("action-delete-domain", conn.get_uri(), vm.get_uuid())
+            self.emit("action-delete-domain", conn.get_uri(), vm.get_connkey())
 
     def _do_delete_conn(self, conn):
         if conn is None:
@@ -555,27 +541,28 @@ class vmmManager(vmmGObjectUI):
 
     def start_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-run-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-run-domain", vm.conn.get_uri(), vm.get_connkey())
 
     def poweroff_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-shutdown-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-shutdown-domain",
+            vm.conn.get_uri(), vm.get_connkey())
 
     def pause_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-suspend-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-suspend-domain", vm.conn.get_uri(), vm.get_connkey())
 
     def resume_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-resume-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-resume-domain", vm.conn.get_uri(), vm.get_connkey())
 
     def close_conn(self, ignore):
         conn = self.current_conn()
@@ -596,8 +583,8 @@ class vmmManager(vmmGObjectUI):
     def vm_row_key(self, vm):
         return vm.get_uuid() + ":" + vm.conn.get_uri()
 
-    def vm_added(self, conn, vmuuid):
-        vm = conn.get_vm(vmuuid)
+    def vm_added(self, conn, connkey):
+        vm = conn.get_vm(connkey)
         if self.vm_row_key(vm) in self.rows:
             return
 
@@ -611,14 +598,14 @@ class vmmManager(vmmGObjectUI):
 
         self._append_vm(model, vm, conn)
 
-    def vm_removed(self, conn, vmuuid):
+    def vm_removed(self, conn, connkey):
         vmlist = self.widget("vm-list")
         model = vmlist.get_model()
 
         parent = self.rows[conn.get_uri()].iter
         for row in range(model.iter_n_children(parent)):
             vm = model[model.iter_nth_child(parent, row)][ROW_HANDLE]
-            if vm.get_uuid() == vmuuid:
+            if vm.get_connkey() == connkey:
                 model.remove(model.iter_nth_child(parent, row))
                 del self.rows[self.vm_row_key(vm)]
                 break
@@ -654,7 +641,7 @@ class vmmManager(vmmGObjectUI):
 
     def _build_row(self, conn, vm):
         if conn:
-            name = conn.get_pretty_desc_inactive(False)
+            name = conn.get_pretty_desc(shorthost=False)
             markup = self._build_conn_markup(conn, name)
             status = ("<span size='smaller'>%s</span>" %
                       conn.get_state_text())
@@ -710,7 +697,43 @@ class vmmManager(vmmGObjectUI):
         self.rows[conn.get_uri()] = model[path]
         return _iter
 
+    def _ensure_conn_descs_dont_collide(self):
+        # By default we only show hostname + hypervisor in the conn label.
+        # So if we have two URIs like qemu+ssh://host and qemu+tcp://host,
+        # we want to add the transport in the description to differentiate
+        connrows = [row for row in self.rows.values() if row[ROW_IS_CONN]]
+        for row in connrows:
+            conn = row[ROW_HANDLE]
+            connsplit = util.uri_split(conn.get_uri())
+            scheme = connsplit[0]
+
+            show_transport = False
+            show_user = False
+
+            for checkrow in connrows:
+                checkconn = checkrow[ROW_HANDLE]
+                if conn is checkconn:
+                    continue
+                checkconnsplit = util.uri_split(checkconn.get_uri())
+                checkscheme = checkconnsplit[0]
+
+                if ((scheme.split("+")[0] == checkscheme.split("+")[0]) and
+                    connsplit[2] == checkconnsplit[2] and
+                    connsplit[3] == checkconnsplit[3]):
+                    show_transport = True
+                    if ("+" in scheme and "+" in checkscheme and
+                        scheme.split("+")[1] == checkscheme.split("+")[1]):
+                        show_user = True
+
+            newname = conn.get_pretty_desc(
+                shorthost=False, show_transport=show_transport,
+                show_user=show_user)
+            if newname != row[ROW_SORT_KEY]:
+                self.conn_state_changed(conn, newname=newname)
+
     def add_conn(self, engine_ignore, conn):
+        # Called from engine.py signal conn-added
+
         # Make sure error page isn't showing
         self.widget("vm-notebook").set_current_page(0)
 
@@ -722,30 +745,13 @@ class vmmManager(vmmGObjectUI):
         conn.connect("resources-sampled", self.conn_row_updated)
         conn.connect("state-changed", self.conn_state_changed)
 
-        # add the connection to the treeModel
         vmlist = self.widget("vm-list")
-        row = self._append_conn(vmlist.get_model(), conn)
-        vmlist.get_selection().select_iter(row)
-
-        # Try to make sure that 2 row descriptions don't collide
-        connrows = []
-        descs = []
-        for row in self.rows.values():
-            if row[ROW_IS_CONN]:
-                connrows.append(row)
-        for row in connrows:
-            descs.append(row[ROW_SORT_KEY])
-
-        for row in connrows:
-            conn = row[ROW_HANDLE]
-            name = row[ROW_SORT_KEY]
-            if descs.count(name) <= 1:
-                continue
-
-            newname = conn.get_pretty_desc_inactive(False, True)
-            self.conn_state_changed(conn, newname=newname)
+        self._append_conn(vmlist.get_model(), conn)
+        self._ensure_conn_descs_dont_collide()
 
     def remove_conn(self, engine_ignore, uri):
+        # Called from engine.py signal conn-removed
+
         model = self.widget("vm-list").get_model()
         parent = self.rows[uri].iter
 
@@ -830,6 +836,29 @@ class vmmManager(vmmGObjectUI):
 
         self.vm_row_updated(vm)
 
+    def set_initial_selection(self, uri):
+        vmlist = self.widget("vm-list")
+        model = vmlist.get_model()
+        it = model.get_iter_first()
+        selected = None
+        while it:
+            key = model.get_value(it, ROW_HANDLE)
+
+            if key.get_uri() == uri:
+                vmlist.get_selection().select_iter(it)
+                return
+
+            if not selected:
+                vmlist.get_selection().select_iter(it)
+                selected = key
+            elif key.get_autoconnect() and not selected.get_autoconnect():
+                vmlist.get_selection().select_iter(it)
+                selected = key
+                if not uri:
+                    return
+
+            it = model.iter_next(it)
+
     def conn_state_changed(self, conn, newname=None):
         row = self.rows[conn.get_uri()]
         if newname:
@@ -930,8 +959,7 @@ class vmmManager(vmmGObjectUI):
             # Popup the vm menu
             vm = model[_iter][ROW_HANDLE]
             self.vmmenu.update_widget_states(vm)
-            self.vmmenu.popup(  # pylint: disable=E1101
-                None, None, None, None, 0, event.time)
+            self.vmmenu.popup(None, None, None, None, 0, event.time)
         else:
             # Pop up connection menu
             conn = model[_iter][ROW_HANDLE]
